@@ -31,20 +31,37 @@ get.eps.T <- function (DLpar, country, meta, ...)
     if (!is.null(meta$ar.phase2) && meta$ar.phase2) 
     {
       args <- list(...)
-      if ('rho.phase2' %in% names(args)) eps <- c(eps[1], eps[2:ldl]-args[['rho.phase2']] * eps[1:(ldl-1)])
+      if ('rho.phase2' %in% names(args) && length(eps) > 1) eps <- c(eps[1], eps[2:ldl]-args[['rho.phase2']] * eps[1:(ldl-1)])
     }
+
+    # Put NAs on eps indexed by meta$indices.outliers[[country]]
+    if (as.character(country) %in% names(meta$indices.outliers))
+    {
+      outlier_indices <- meta$indices.outliers[[as.character(country)]]
+      if (!is.null(meta$ar.phase2) && meta$ar.phase2)
+        outlier_indices <- sort(unique(c(outlier_indices, outlier_indices+1)))
+      
+      eps <- eps[-outlier_indices]
+    }
+      
     return (eps)
 }
 
 get_eps_T_all <- function (mcmc, ...) {
   suppl.T <- if(!is.null(mcmc$meta$suppl.data$regions)) mcmc$meta$suppl.data$T_end else 0
 	eps_Tc <- matrix(NA, mcmc$meta$T_end-1 + suppl.T, mcmc$meta$nr_countries)
-    for (country in mcmc$meta$id_DL){
-    	theta <- c((mcmc$U_c[country]-mcmc$Triangle_c4[country])*exp(mcmc$gamma_ci[country,])/                                     
-                                sum(exp(mcmc$gamma_ci[country,])), mcmc$Triangle_c4[country], mcmc$d_c[country])
-        eps_Tc[mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1), country] <- get.eps.T(theta, country, mcmc$meta, ...)
-        }
-    return(eps_Tc)
+  for (country in mcmc$meta$id_DL){
+    theta <- c((mcmc$U_c[country]-mcmc$Triangle_c4[country])*exp(mcmc$gamma_ci[country,])/                                     
+      sum(exp(mcmc$gamma_ci[country,])), mcmc$Triangle_c4[country], mcmc$d_c[country])
+    idx <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1)
+    raw.outliers <- mcmc$meta$indices.outliers[[as.character(country)]]
+    if (!is.null(mcmc$meta$ar.phase2) && mcmc$meta$ar.phase2) 
+      raw.outliers <- sort(unique(c(raw.outliers, raw.outliers+1)))
+    idx <- setdiff(idx, raw.outliers)
+    eps_Tc[idx, country] <- get.eps.T(theta, country, mcmc$meta, ...)
+  }
+	
+  return(eps_Tc)
 }
 
 find.lambda.for.one.country <- function(tfr, T_end, annual = FALSE) {
@@ -199,6 +216,28 @@ find.tau.lambda.and.DLcountries <- function(tfr_matrix, min.TFRlevel.for.start.a
          ))
 }
 
+
+find.raw.data.outliers <- function(raw.tfr, iso.unbiased, max.drop=1, max.increase=1, 
+                                   source.col.name="source"){
+  if(is.null(iso.unbiased)) return(NULL)
+  MAX_TFR <-  10
+  outliers  <- list()
+  for (iso.code in iso.unbiased){
+    vr.ts <- raw.tfr[(raw.tfr[,source.col.name]=="VR") & (raw.tfr$country_code==iso.code),]
+    if (nrow(vr.ts) == 0) next
+    year.ts <- round(vr.ts$year - 0.01)
+    tfr.ts <- vr.ts$tfr
+    annual.tfr.diff <- diff(tfr.ts) / diff(year.ts)
+    outlier <- (annual.tfr.diff > max.increase & abs(annual.tfr.diff) < MAX_TFR) |
+      (annual.tfr.diff < -max.drop & abs(annual.tfr.diff) < MAX_TFR)
+    outlier[is.na(outlier)] <- FALSE
+    year.outlier <- year.ts[outlier]
+    # for all year in year.outlier, TFR of year->year+1 is an outlier
+    if (length(year.outlier) > 0) outliers[[as.character(iso.code)]] <- year.outlier
+  }
+  return (outliers)
+}
+
 mcmc.meta.ini <- function(...,
 						U.c.low,
 						start.year=1950, present.year=2020, 
@@ -206,9 +245,10 @@ mcmc.meta.ini <- function(...,
 						proposal_cov_gammas = NULL, # should be a list with elements 'values' and 'country_codes'
 						verbose=FALSE, uncertainty=FALSE,
 						my.tfr.raw.file=NULL, 
-						ar.phase2=FALSE) {
+						ar.phase2=FALSE, iso.unbiased=NULL, source.col.name = "source") {
 	# Initialize meta parameters - those that are common to all chains.
-	args <- list(...)
+    
+  args <- list(...)
 	mcmc.input <- list()
 	for (arg in names(args)) mcmc.input[[arg]] <- args[[arg]]
 	mcmc.input$U.c.low.base <- U.c.low
@@ -222,7 +262,8 @@ mcmc.meta.ini <- function(...,
 										verbose=verbose)
 	meta <- do.meta.ini(mcmc.input, tfr.with.regions,  
 						proposal_cov_gammas=proposal_cov_gammas, verbose=verbose, 
-						uncertainty=uncertainty, my.tfr.raw.file=my.tfr.raw.file, ar.phase2=ar.phase2)
+						uncertainty=uncertainty, my.tfr.raw.file=my.tfr.raw.file, ar.phase2=ar.phase2, 
+						iso.unbiased=iso.unbiased, source.col.name = source.col.name)
 	return(structure(c(mcmc.input, meta), class='bayesTFR.mcmc.meta'))
 }
 	
@@ -230,7 +271,7 @@ mcmc.meta.ini <- function(...,
 do.meta.ini <- function(meta, tfr.with.regions, proposal_cov_gammas = NULL, 
 						use.average.gammas.cov=FALSE, burnin=200, verbose=FALSE, uncertainty=FALSE, 
 						my.tfr.raw.file=NULL, 
-						ar.phase2=FALSE) {
+						ar.phase2=FALSE, iso.unbiased=NULL, source.col.name = "source") {
   results_tau <- find.tau.lambda.and.DLcountries(tfr.with.regions$tfr_matrix, annual = meta$annual.simulation,
 	                                               suppl.data=tfr.with.regions$suppl.data)
 	tfr_matrix_all <- tfr.with.regions$tfr_matrix_all
@@ -335,6 +376,33 @@ do.meta.ini <- function(meta, tfr.with.regions, proposal_cov_gammas = NULL,
 	  index <- 1:nrow(output$raw_data.original)
 	  country.ind.by.year <- list()
 	  ind.by.year <- list()
+	  
+	  # Defining a list containing indices of outliers by countries 
+	  # (only countries with outliers will have an entry in the list)
+	  # outliers are those where the annual delta is outside of the interval (meta$raw.outliers[1], meta$raw.outliers[2])
+	  
+	  year.outliers <- find.raw.data.outliers(rawTFR, iso.unbiased, source.col.name = source.col.name,
+	                                          max.drop = -meta$raw.outliers[1], max.increase = meta$raw.outliers[2])
+	  indices.outliers <- list()
+	  yearly.outliers <- list()
+	  for (iso.code in names(year.outliers))
+	  {
+	    country.index <-  get.country.object(country = as.numeric(iso.code), meta = output)$index
+	    indices <- year.outliers[[iso.code]] - meta$start.year + 1
+	    # We may remove outliers in Phase I?
+	    indices <- indices[indices > results_tau$start_c[country.index]]
+	    indices <- indices[indices <= results_tau$lambda_c[country.index]]
+	    if (length(indices) > 0) {
+	      indices.outliers[[as.character(country.index)]] <- indices
+	      for (ind in indices) {
+	        if (as.character(ind) %in% year.outliers) yearly.outliers[[as.character(ind)]] <- c(yearly.outliers[[as.character(ind)]], country.index)
+	        else yearly.outliers[[as.character(ind)]] <- country.index
+	      }
+	        
+	    }
+	  }
+	  output$indices.outliers <- indices.outliers
+	  output$yearly.outliers <- yearly.outliers
 	  if (meta$annual.simulation)
 	  {
 	    for (year in meta$start.year:meta$present.year)
@@ -508,7 +576,7 @@ mcmc.ini <- function(chain.id, mcmc.meta, iter=100,
 mcmc.meta.ini.extra <- function(mcmc.set, countries=NULL, my.tfr.file = NULL, 
 									my.locations.file=NULL, burnin = 200, verbose=FALSE, uncertainty=FALSE, 
 									my.tfr.raw.file=ifelse(uncertainty, file.path(find.package("bayesTFR"), "data", "rawTFR.csv"), NULL),
-									average.gammas.cov = TRUE) {
+									average.gammas.cov = TRUE, iso.unbiased=NULL) {
 	update.regions <- function(reg, ereg, id.replace, is.new, is.old) {
 		nreg <- list()
 		for (name in c('code', 'area_code', 'country_code')) {
